package de.cubeisland.AntiGuest.prevention.preventions;

import de.cubeisland.AntiGuest.prevention.Prevention;
import de.cubeisland.AntiGuest.prevention.PreventionPlugin;
import gnu.trove.map.hash.TObjectLongHashMap;
import org.bukkit.configuration.Configuration;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.EventPriority;
import org.bukkit.event.player.AsyncPlayerChatEvent;

import java.util.concurrent.TimeUnit;

/**
 * Prevents spamming
 *
 * @author Phillip Schichtel
 */
public class SpamPrevention extends Prevention
{
    private long spamLockDuration;
    private TObjectLongHashMap<String> chatTimestamps;

    public SpamPrevention(PreventionPlugin plugin)
    {
        super("spam", plugin);
        setEnableByDefault(true);
        setEnablePunishing(true);
    }

    @Override
    public String getConfigHeader()
    {
        return super.getConfigHeader() +
                "Configuration info:\n" +
                "    lockDuration: the time in seconds a player has to wait between messages";
    }

    @Override
    public Configuration getDefaultConfig()
    {
        Configuration config = super.getDefaultConfig();

        config.set("lockDuration", 2);

        return config;
    }
    
    @Override
    public void enable()
    {
        super.enable();
        this.spamLockDuration = TimeUnit.SECONDS.toMillis(getConfig().getLong("lockDuration"));
        this.chatTimestamps = new TObjectLongHashMap<String>();
    }

    @Override
    public void disable()
    {
        super.disable();
        this.chatTimestamps.clear();
        this.chatTimestamps = null;
    }

    @EventHandler(priority = EventPriority.LOWEST, ignoreCancelled = true)
    public void chat(AsyncPlayerChatEvent event)
    {
        final Player player = event.getPlayer();
        if (!can(player))
        {
            if (isChatLocked(player))
            {
                prevent(event, player);
            }
            else
            {
                setChatLock(player);
            }
        }
    }

    private synchronized void setChatLock(final Player player)
    {
        this.chatTimestamps.put(player.getName(), System.currentTimeMillis() + this.spamLockDuration);
    }
    
    private synchronized boolean isChatLocked(final Player player)
    {
        final long nextPossible = this.chatTimestamps.get(player.getName());
        if (nextPossible == 0)
        {
            return false;
        }

        final long currentTime = System.currentTimeMillis();
        if (nextPossible < currentTime)
        {
            return false;
        }
        this.chatTimestamps.remove(player.getName());
        return true;
    }
}
